#!/usr/bin/python
#
# Copyright 2018-2021 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from marshmallow import fields, validate

from polyaxon.schemas.base import BaseCamelSchema, BaseConfig


class BaseIterationSchema(BaseCamelSchema):
    iteration = fields.Int()
    configs = fields.List(
        fields.List(fields.Raw(), validate=validate.Length(equal=2)), allow_none=True
    )
    metrics = fields.List(
        fields.List(fields.Raw(), validate=validate.Length(equal=2)), allow_none=True
    )

    @staticmethod
    def schema_config():
        return BaseIterationConfig


class BaseIterationConfig(BaseConfig):
    SCHEMA = BaseIterationSchema

    def __init__(self, iteration, configs, metrics):
        self.iteration = iteration
        self.configs = configs
        self.metrics = metrics
